/*
  Wattmètre compteur de vélo v2.1
        affiche la puissance P, le courant I, la tension U ou la capacité consommée en Wh sur un compteur de vélo filaire

  Composants :
     - 1 capteur de courant ACS711EX +- 31A
     - 1 pont diviseur de tension pour la mesure de tension
     - 1 interrupteur pour choisir l'affichage de la puissance P ou du courant I sur le compteur
     - 1 bouton poussoir pour afficher la tension U ou la capacité consommée en Wh sur le compteur
     - 1 ATMEL ATTINY 85 (fonctionnement sur oscillateur interne 8 MHz)
            Pin 1 is /RESET
                            +-\/-+
            Ain0 (D5) PB5  1|    |8  Vcc
            Ain3 (D3) PB3  2|    |7  PB2 (D2) Ain1
            Ain2 (D4) PB4  3|    |6  PB1 (D1) pwm1
                       GND 4|    |5  PB0 (D0) pwm0

  Versions :
      1.0 : version simple Arduino Pro Mini avec delay
      2.0 : version ATtiny85 avec timer
      2.1 : ajout d'une moyenne sur le calcul de deltaI
*/



// à décommenter pour débuggage avec SoftwareSerial (ne fonctionne qu'à 8MHz)
/*
  #include "SoftwareSerial.h"
  const int Rx = 1;
  const int Tx = 0;
  SoftwareSerial mySerial(Rx, Tx);
*/

const int PIN_READ_MODE = 0;    // à activer pour afficher P au lieu de I
const int PIN_READ_SELECT = 1;  // à activer pour afficher U ou Wh
const int PIN_SPEED = 2;        // commande de l'affichage du compteur
const int PIN_VOLTAGE = A3;     // capteur de tension
const int PIN_CURRENT = A2;     // capteur de courant

boolean readMode = 0;     // HIGH = affichage de I; LOW = affichage de P
boolean readSelect = 0;   // HIGH = affichage de I ou P; LOW = affichage de Wh ou U
float U = 0;          // Voltage
float I = 0;          // Current
float deltaI = 0;     // pour la mise à 0 de I à chaque reset
float Umoy = 0;       // tension moyenne sur i valeurs
float Imoy = 0;       // courant moyen sur i valeurs
float Pmoy = 0;       // Puissance moyenne sur i valeurs
float Ah = 0;         // Battery capacity in Ah
float Wh = 0;         // Battery capacity in Wh
float diamRoue = 2;   // diamètre arbitraire en mètre, à rentrer dans le compteur
int prescaler = 512;  // prescaler du timer
float freqATtiny85 = 8 * pow(10, 6);  // fréquence de l'oscillateur interne du microcontrôleur ATtiny85
float counterTimerCorrectif = 1.014;  // coefficient correctif du fait de l'imprécision de la fréquence de l'horloge interne du microcontrôleur ATtiny85
float counterTimer = 0;
float counterTimerLimit = 0;
boolean delaySpeedSensorCalculation = false;

int i = 0;          // nb déchantillons pour calculer les moyennes tension, courant et puissance
float kU = (110 + 10) / 10.0 * 0.983;  // pont diviseur de tension pour la mesure de tension batterie : R1 = 110k et R2 = 10k
float kI = 1 / 0.068;  // Capteur de courant ACS711EX : 2.5 V à vide et 0.068V/A pour la version +-31A; 0.136V/A pour la version +-15.5A

void setup() {

  // à décommenter pour débuggage avec SoftwareSerial
  /*
    pinMode(Rx, INPUT);
    pinMode(Tx, OUTPUT);
    mySerial.begin(9600); // send serial data at 9600 bits/sec
  */

  // Timer 1
  TCCR1 = bit (CTC1);   // Clear Timer/Counter on Compare Match
  //TCCR1 |= bit (CS10) | bit (CS13);  // prescaler of 256
  TCCR1 |= bit (CS11) | bit (CS13);  // prescaler: 512
  //TCCR1 |= bit (CS10) | bit (CS11) | bit (CS13);  // prescaler: 1024
  //TCCR1 |= bit (CS10) | bit (CS11) | bit (CS12) | bit (CS13);  // prescaler: 16384
  OCR1C = 1; // what to count to (zero-relative)
  TIMSK = bit (OCIE1A); // interrupt on compare

  pinMode(PIN_READ_MODE, INPUT_PULLUP);  // à commenter si débuggage avec SoftwareSerial
  pinMode(PIN_READ_SELECT, INPUT_PULLUP);
  pinMode(PIN_SPEED, OUTPUT);  // sets the digital pin as output
  pinMode(PIN_VOLTAGE, INPUT); // Voltage pin
  pinMode(PIN_CURRENT, INPUT); // Current pin

  for (int i = 0; i <= 9; i++) { // mesure deltaI pour étalonnage capteur de courant
    deltaI = deltaI + analogRead(PIN_CURRENT);
  }
  deltaI = deltaI / 10.0;  // calcul deltaI moyen
}

void loop() {

  // moyenne des valeurs de tension (U) et courant (I) sur 10 valeurs
  U = 0;
  I = 0;
  for (int i = 0; i <= 9; i++) {  // moyenne tension, intensité et puissance sur 10 valeurs
    U = U + analogRead(PIN_VOLTAGE) * (5.0 / 1023.0) * kU; // Calcul tension
    I = I + (analogRead(PIN_CURRENT) - deltaI) * (5.0 / 1023.0) * kI;   // Calcul courant
  }

  Umoy = U / 10.0;     // calcul tension moyenne
  Imoy = I / 10.0;     // calcul courant moyen
  Pmoy = Umoy * Imoy;  // calcul puissance

  readMode = digitalRead(PIN_READ_MODE);
  readSelect = digitalRead(PIN_READ_SELECT);
  if (readMode == HIGH && readSelect == HIGH && Imoy > 0) {
    delaySpeedSensorCalculation = true;
    counterTimerLimit = (diamRoue * 3.6 / (Imoy * 10)) / (2 * 2 * prescaler / freqATtiny85) * counterTimerCorrectif;  // si Imoy > 0A, affichage courant : 1A = 10km/h; 2 * prescaler car OCR1C = 1
  }
  else if (readMode == LOW && readSelect == HIGH && Pmoy > 0) {
    delaySpeedSensorCalculation = true;
    counterTimerLimit = (diamRoue * 3.6 / (Pmoy / 10.0)) / (2 * 2 * prescaler / freqATtiny85) * counterTimerCorrectif;  // si Pmoy > 0W, affichage puissance : 10W = 1km/h
  }
  else if (readMode == HIGH && readSelect == LOW && Wh > 0) {
    delaySpeedSensorCalculation = true;
    counterTimerLimit = (diamRoue * 3.6 / (Wh / 10.0)) / (2 * 2 * prescaler / freqATtiny85) * counterTimerCorrectif * counterTimerCorrectif;  // si Wh > 0 Wh, affichage capacité batterie : 10Wh = 1km/h
  }
  else if (readMode == LOW && readSelect == LOW && Umoy > 0) {
    delaySpeedSensorCalculation = true;
    counterTimerLimit = (diamRoue * 3.6 / Umoy) / (2 * 2 * prescaler / freqATtiny85) * counterTimerCorrectif;  // si Umoy > 0V, affichage tension batterie : 1V = 1km/h
  }
  else {
    delaySpeedSensorCalculation = false;  // sinon, pas de calcul de counterTimerLimit
  }

  // à décommenter pour débuggage avec SoftwareSerial
  /*
    mySerial.print(Umoy);
    mySerial.print(" ");
    mySerial.print(Imoy);
    mySerial.print(" ");
    mySerial.print(Pmoy);
    mySerial.print(" ");
    mySerial.println(Wh);
  */
}

ISR (TIMER1_COMPA_vect) {
  counterTimer++;
  if (counterTimer >= counterTimerLimit && delaySpeedSensorCalculation == true) {
    Wh = Wh + Umoy * Imoy * counterTimer * prescaler * 2 / (freqATtiny85 * 3600.0); // calcul Wh
    digitalWrite (PIN_SPEED, ! digitalRead (PIN_SPEED));
    counterTimer = 0;
  }
}
